'''
Created on 04.05.2010

@author: hedger
'''

from numpy import * 
import itertools

set_printoptions(precision = 2, 
                 nanstr =' - ')

class Criteria():
    cost = 0
    values = []
    def __init__(self, cost, values):
        self.cost, self.values = cost, values
        
    def __repr__(self):
        return "<Criteria cost:%d vals:%s>" % (self.cost, self.values)
    
    def compare(self, val1, val2):
        valSet = set(self.values)
        if (val1 in valSet) and (val2 in valSet):
            return cmp(self.values.index(val1) - self.values.index(val2), 0) * self.cost
        else: 
            raise Exception("Element in (%s,%s) is not defined!" % (val1, val2)) 
    
    def loadFromString(self, criteriaString):
        data = criteriaString.split(';')
        assert len(data) > 1
        self.cost = int(data[0])
        self.values = data[1:]
        
    @staticmethod
    def createFromString(criteriaString):
        data = criteriaString.split(';')
        assert len(data) > 1
        return Criteria(int(data[0]), data[1:])

class Project():
    id = 0
    values = []
    def __init__(self, criteriaList, id, values):
        self.criteriaList, self.id, self.values = criteriaList, id, values
        
    def __repr__(self):
        return "<Project id:%d vals:%s>" % (self.id, self.values)
    
    def getValue(self, id):
        return self.values[id]
    
    def compare(self, cmpProject):
        Prate = Nrate = 0 
        assert(cmpProject.id != self.id)
        for i in range(len(self.values)):
            criteria = self.criteriaList[i]
            # print "Using %s for param#%d" % (criteria, i) 
            cmpResult = criteria.compare(self.getValue(i), cmpProject.getValue(i))
            if (cmpResult > 0):
                Prate += cmpResult
            else:
                Nrate -= cmpResult 
        return (Prate, Nrate)
    
    def preferanceIndex(self, cmpProject):
        if self.id == cmpProject.id: #FIXME: rewrite main diag fix
            return NaN
        (Prate, Nrate) = self.compare(cmpProject)
        try:
            prefIndex = float(Prate) / Nrate
        except ZeroDivisionError:
            return inf
        if prefIndex > 1:
            return prefIndex
        else:
            return NaN

    def loadFromString(self, projectString):
        data = projectString.split(';')
        assert len(data) > 1
        self.id = int(data[0])
        self.values = data[1:]
        assert (len(self.values) == len(self.criteriaList))

    @staticmethod
    def createFromString(projectString, criteriaList):
        data = projectString.split(';')
        assert len(data) > 1
        assert (len(data[1:]) == len(criteriaList))
        return Project(criteriaList, int(data[0]), data[1:])

def checkPairs(projects):
    for pair in itertools.combinations(projects, 2): # permutations
        p1, p2 = pair[0], pair[1]
        print "Cmp: %d:%d -> %s" % (p1.id, p2.id, p1.compare(p2))

def buildPreference(projects):
    prefMatrix = zeros((len(projects), len(projects)))
    #for pair in itertools.permutations(projects, 2): # permutations
    for pair in itertools.product(projects, repeat=2): # permutations
        p1, p2 = pair[0], pair[1]
        #print "PId: %d:%d -> %s" % (p1.id, p2.id, p1.preferanceIndex(p2))
        #print "PIdx: %d:%d -> %s" % (projects.index(p1), projects.index(p2), p1.preferanceIndex(p2))
        prefMatrix[projects.index(p1), projects.index(p2)] = p1.preferanceIndex(p2)
    return prefMatrix

def cutPrefMatrix(prefMatrix, limit):
    newMatrix = prefMatrix.copy()
    for ((x,y), val) in ndenumerate(newMatrix):
        if limit > val:
            newMatrix[x,y] = NaN
    return newMatrix

def autocutPrefMatrix(prefMatrix, stepsNum, projects):
    steps = sorted(set(filter(lambda x: not(isnan(x)) and (x != inf), prefMatrix.flatten())))
    for step in steps:
        cutMatrix = cutPrefMatrix(prefMatrix, step)
        print "Cut @ %s:\nMatrix:\n" % step
        print cutMatrix
        print "\nGraph code:\n" % step
        print graphMatrix(cutMatrix, projects)
        print "\n\n", "="*40
    
def graphMatrix(prefMatrix, projects):
    def createStates():
        states = []
        for prj in projects:
            states.append(("n%s" % prj.id, prj.id))
        return states
    
    def createConnections():
        connections = [] 
        i = j = 0
        for row in prefMatrix:
            for item in row:
                #print "Testing '%s' @ %d:%d, %s" % (item, i, j, not(isnan(item)))
                if not(isnan(item)):
                    #print "Linking @ %d:%d" %(i,j)
                    connections.append(("n%s" % projects[i].id, "n%s" % projects[j].id,))
                j += 1
            i += 1
            j = 0
        return connections
    
    states = createStates()
    conns = createConnections()
    dgCode = "digraph projects {\n"
    for state in states:
        dgCode += "   %s [label = \"%s\"];\n" % (state[0], state[1])
    for conn in conns:
        dgCode += "   %s -> %s;\n" % (conn[0], conn[1])
    dgCode += "}"
    return dgCode
        
if __name__ == '__main__':
    """
    a = Criteria()
    a.loadFromString('-4;Sl;Sm;Sh')
    b = Criteria()
    b.loadFromString('5;Nl;Nh')
    crList = [a, b]
    prj1, prj2 = Project(crList), Project(crList)
    prj1.loadFromString('1;Sm;Nh')
    prj2.loadFromString('2;Sh;Nl')
    print prj1.compare(prj2)
    """